<?php

namespace Instant\Credit\Controller\Redirect;

use Instant\Credit\Api\ApiException;
use Instant\Credit\Model\Config\Source\StoreType;
use Instant\Credit\Model\Enum\ResultType;
use Instant\Credit\Model\ResourceModel\Transactions;
use Magento\Framework\Api\FilterBuilder;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\View\Result\PageFactory;
use Magento\Framework\App\ActionInterface;
use Magento\Sales\Api\OrderRepositoryInterface;

/**
 * Class Index
 * @package Instant\Credit\Controller\Redirect
 * @author Orenyi
 */
class Index implements ActionInterface
{
    /**
     * @var PageFactory
     */
    private $pageFactory;

    /**
     * @var \Magento\Checkout\Model\Session
     */
    private $checkoutSession;

    /**
     * @var \Instant\Credit\Api\Data\CreditInterfaceFactory
     */
    protected $credit;

    /**
     * @var \Instant\Credit\Api\Data\RequestInterfaceFactory
     */
    protected $request;

    /**
     * @var \Magento\Customer\Model\CustomerFactory
     */
    protected $customer;

    /**
     * @var \Instant\Credit\Api\Data\AddressInterfaceFactory
     */
    protected $address;

    /**
     * @var \Instant\Credit\Api\Data\EndUserInterfaceFactory
     */
    protected $endUser;

    /**
     * @var \Magento\Framework\Api\SearchCriteriaInterface
     */
    protected $searchCriteriaBuilder;

    /**
     * @var \Magento\Framework\Api\FilterBuilder
     */
    protected $filterBuilder;

    /**
     * @var \Magento\Framework\Api\Search\FilterGroupBuilder
     */
    protected $filterGroupBuilder;

    /**
     * @var OrderRepositoryInterface
     */
    protected $orderRepository;

    /**
     * @var \Instant\Credit\Api\Data\HistoricOrderInterfaceFactory
     */
    protected $historic;

    /**
     * @var \Instant\Credit\Logger\InstantCredit
     */
    protected $logger;

    /**
     * @var \Magento\Framework\HTTP\PhpEnvironment\RemoteAddress
     */
    protected $remoteAddress;
    /**
     * @var \Magento\Framework\Controller\ResultFactory
     */
    private $resultFactory;
    /**
     * @var \Magento\Store\Model\StoreManagerInterface
     */
    private $storeManager;
    /**
     * @var \Instant\Credit\Api\Data\TransactionsInterfaceFactory
     */
    private $transactionsInterfaceFactory;
    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    private $scopeConfig;
    /**
     * @var \Magento\Framework\Serialize\Serializer\Json
     */
    private $serializer;
    /**
     * @var Transactions
     */
    private $transactionResource;
    /**
     * @var \Instant\Credit\Api\Data\CartItemInterfaceFactory
     */
    private $cartItemFactory;
    /**
     * @var \Magento\Catalog\Model\ResourceModel\Product\CollectionFactory
     */
    private $productCollectionFactory;
    /**
     * @var \Magento\Catalog\Model\ResourceModel\Category\CollectionFactory
     */
    private $categoryCollectionFactory;
    /**
     * @var \Magento\Sales\Model\ResourceModel\Order
     */
    private $resourceOrder;
    /**
     * @var \Magento\Framework\Message\ManagerInterface
     */
    private $messageManager;
    /**
     * @var \Magento\Sales\Model\ResourceModel\Order\CollectionFactory
     */
    private $orderCollection;
    /**
     * @var \Magento\Sales\Model\ResourceModel\Order
     */
    private $orderResource;
    /**
     * @var \Magento\Customer\Api\CustomerRepositoryInterface
     */
    private $customerRepositoryInterface;
    /**
     * @var \Magento\Customer\Model\ResourceModel\Customer\CollectionFactory
     */
    private $customerCollectionFactory;
    /**
     * @var \Magento\Sales\Api\OrderManagementInterface
     */
    private $orderManagement;

    /**
     * Index constructor.
     *
     * @param PageFactory $pageFactory
     * @param \Magento\Checkout\Model\Session $checkoutSession
     * @param \Instant\Credit\Api\Data\CreditInterfaceFactory $credit
     * @param \Instant\Credit\Api\Data\RequestInterfaceFactory $request
     * @param \Magento\Customer\Api\CustomerRepositoryInterface $customer
     * @param \Instant\Credit\Api\Data\AddressInterfaceFactory $address
     * @param \Instant\Credit\Api\Data\EndUserInterfaceFactory $endUser
     * @param \Magento\Framework\Api\SearchCriteriaInterface $searchCriteriaBuilder
     * @param \Magento\Framework\Api\Search\FilterGroupBuilder $filterGroupBuilder
     * @param OrderRepositoryInterface $orderRepository
     * @param \Instant\Credit\Api\Data\HistoricOrderInterfaceFactory $historic
     * @param \Instant\Credit\Logger\InstantCredit $logger
     * @param \Magento\Framework\HTTP\PhpEnvironment\RemoteAddress $remoteAddress
     * @param \Magento\Framework\Controller\ResultFactory $resultFactory
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \Instant\Credit\Api\Data\TransactionsInterfaceFactory $transactionsInterfaceFactory
     * @param Transactions $transactionResource
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     * @param \Magento\Framework\Serialize\Serializer\Json $serializer
     * @param \Instant\Credit\Api\Data\CartItemInterfaceFactory $cartItemFactory
     * @param \Magento\Catalog\Model\ResourceModel\Product\CollectionFactory $productCollectionFactory
     * @param \Magento\Catalog\Model\ResourceModel\Category\CollectionFactory $categoryCollectionFactory
     * @param \Magento\Sales\Model\ResourceModel\Order $resourceOrder
     * @param \Magento\Framework\Message\ManagerInterface $messageManager
     * @param \Magento\Sales\Model\ResourceModel\Order\CollectionFactory $orderCollection
     * @param \Magento\Sales\Model\ResourceModel\Order $orderResource
     * @param \Magento\Customer\Model\ResourceModel\Customer\CollectionFactory $customerCollectionFactory
     * @param \Magento\Sales\Api\OrderManagementInterface $orderManagement
     */
    public function __construct(
        PageFactory $pageFactory,
        \Magento\Checkout\Model\Session $checkoutSession,
        \Instant\Credit\Api\Data\CreditInterfaceFactory $credit,
        \Instant\Credit\Api\Data\RequestInterfaceFactory $request,
        \Magento\Customer\Api\CustomerRepositoryInterface $customer,
        \Instant\Credit\Api\Data\AddressInterfaceFactory $address,
        \Instant\Credit\Api\Data\EndUserInterfaceFactory $endUser,
        \Magento\Framework\Api\SearchCriteriaInterface $searchCriteriaBuilder,
        \Magento\Framework\Api\Search\FilterGroupBuilder $filterGroupBuilder,
        \Magento\Sales\Api\OrderRepositoryInterface $orderRepository,
        \Instant\Credit\Api\Data\HistoricOrderInterfaceFactory $historic,
        \Instant\Credit\Logger\InstantCredit $logger,
        \Magento\Framework\HTTP\PhpEnvironment\RemoteAddress $remoteAddress,
        \Magento\Framework\Controller\ResultFactory $resultFactory,
        \Magento\Store\Model\StoreManagerInterface $storeManager,
        \Instant\Credit\Api\Data\TransactionsInterfaceFactory $transactionsInterfaceFactory,
        \Instant\Credit\Model\ResourceModel\Transactions $transactionResource,
        \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig,
        \Magento\Framework\Serialize\Serializer\Json $serializer,
        \Instant\Credit\Api\Data\CartItemInterfaceFactory $cartItemFactory,
        \Magento\Catalog\Model\ResourceModel\Product\CollectionFactory $productCollectionFactory,
        \Magento\Catalog\Model\ResourceModel\Category\CollectionFactory $categoryCollectionFactory,
        \Magento\Sales\Model\ResourceModel\Order $resourceOrder,
        \Magento\Framework\Message\ManagerInterface $messageManager,
        \Magento\Sales\Model\ResourceModel\Order\CollectionFactory $orderCollection,
        \Magento\Sales\Model\ResourceModel\Order $orderResource,
        \Magento\Customer\Model\ResourceModel\Customer\CollectionFactory $customerCollectionFactory,
        \Magento\Sales\Api\OrderManagementInterface $orderManagement
    ) {
        $this->pageFactory = $pageFactory;
        $this->checkoutSession = $checkoutSession;
        $this->credit = $credit;
        $this->request = $request;
        $this->customer = $customer;
        $this->address = $address;
        $this->endUser = $endUser;
        $this->searchCriteriaBuilder = $searchCriteriaBuilder;
        $this->filterGroupBuilder = $filterGroupBuilder;
        $this->orderRepository = $orderRepository;
        $this->historic = $historic;
        $this->logger = $logger;
        $this->remoteAddress = $remoteAddress;
        $this->resultFactory = $resultFactory;
        $this->storeManager = $storeManager;
        $this->transactionsInterfaceFactory = $transactionsInterfaceFactory;
        $this->scopeConfig = $scopeConfig;
        $this->serializer = $serializer;
        $this->transactionResource = $transactionResource;
        $this->cartItemFactory = $cartItemFactory;
        $this->productCollectionFactory = $productCollectionFactory;
        $this->categoryCollectionFactory = $categoryCollectionFactory;
        $this->resourceOrder = $resourceOrder;
        $this->messageManager = $messageManager;
        $this->orderCollection = $orderCollection;
        $this->orderResource = $orderResource;
        $this->customerCollectionFactory = $customerCollectionFactory;
        $this->orderManagement = $orderManagement;
    }

    /**
     * @return \Magento\Framework\App\ResponseInterface|\Magento\Framework\Controller\ResultInterface|\Magento\Framework\View\Result\Page
     */
    public function execute()
    {
        $order = $this->checkoutSession->getLastRealOrder();

        if (count($order->getData()) == 0) {
            return $this->redirect('/');
        }

        try {

            /** @var \Instant\Credit\Model\Api\Data\EndUserApi $endUser */
            $endUser = $this->getEndUser($order);

            /** @var \Instant\Credit\Model\Api\Data\CreditApi $creditRequest */
            $creditRequest = $this->credit->create()
                ->setOrderId($order->getIncrementId() . '_' . $order->getStoreId())
                ->setCustom('')
                ->setAmount($this->formatAmount($order->getGrandTotal()))
                ->setCurrency('EUR')
                ->setCountry('ES')
                ->setSuccessURL($this->getUrl('instantcredit/success'))
                ->setFailURL($this->getUrl('instantcredit/fail'))
                ->setNotificationUrl($this->getUrl('instantcredit/callback'))
                ->setEndUser($endUser);
               /** ->setCart($this->getCartItems($order));**/

            /** @var \Instant\Credit\Api\RequestInterface $request */
            $request = $this->request->create();

            $doRequest = $request->doRequest($creditRequest);
            if ($doRequest['response']['code'] === 500) {
                // Can't do credit request then we cancel manually
                $this->orderManagement->cancel($order->getId());
            }

            $response = $doRequest['response']['data'];
            if (!isset($response['url'])) {
                $this->logger->error($this->serializer->serialize($doRequest));
                throw new ApiException(
                    'There was an error while doing your request. Can\'t obtain URL redirection to Instant Credit.'
                );
            }
            $urlCreditRequest = $response['url'];

            if ($response['result'] === ResultType::FAIL) {
                $this->messageManager->addErrorMessage(__('An error has occurred trying to connect to Instant Credit'));
                throw new ApiException('Credit request returns FAIL. Description : ' . $response['resultDescription']);
            }

            if ($response['resultType'] === ResultType::SUCCESSFUL) {

                if ($urlCreditRequest) {
                    //$page = $this->pageFactory->create();
                    /** @var \Magento\Framework\View\Element\Template $block */
                    //$block = $page->getLayout()->getBlock('ic.redirect.template');
                    //$block->setData('url_redirect', $urlCreditRequest);

                    //return $page;
                    return $this->redirect($urlCreditRequest);
                }
            }
        } catch (\Exception $e) {
            $this->logger->critical($e->getMessage());
            if (isset($creditRequest)) {
                $this->logger->error($this->serializer->serialize($creditRequest->getData()));
            }

            $this->messageManager->addErrorMessage(__('An error has occurred trying to connect to Instant Credit. Try it later.'));
            $this->checkoutSession->restoreQuote(); // We restore quote, cause order has not finished successfully
        }

        return $this->redirect('/');
    }


    /**
     * @param $status
     * @return string
     */
    protected function getInstantCreditStatus($status)
    {
        switch($status) {
            case 'complete':
            case 'closed':
                $icStatus = 'AUTHORIZED';
                break;
            default:
                $icStatus = 'IN_PROCESS';
                break;
        }

        return $icStatus;
    }

    /**
     * @param $id
     * @return \Magento\Framework\DataObject
     */
    private function getCustomerById($id)
    {
        return $this->customerCollectionFactory->create()
            ->addFieldToFilter('entity_id', $id)
            ->getFirstItem();
    }

    /**
     * @param \Magento\Sales\Model\Order $order
     * @return \Instant\Credit\Model\Api\Data\CartItemApi[]
     */
    protected function getCartItems(\Magento\Sales\Model\Order $order)
    {
        $cartItems = [];

        $shippingAmount = $order->getShippingAmount();

        /** @var \Magento\Sales\Api\Data\OrderItemInterface[] $items */
        $items = $order->getAllVisibleItems();
        $storeType = $this->scopeConfig->getValue('payment/instantcredit/store_type');

        foreach ($items as $item) {
            $arrItem = [];
            /** @var \Instant\Credit\Model\Api\Data\CartItemApi $cartItem */
            $cartItem = $this->cartItemFactory->create();
            if (intval($storeType) === StoreType::VALUE_ELECTRONICS) {
                $cartItem->setCategory(strtoupper(StoreType::LABEL_ELECTRONICS));
                $cartItem->setBrand($item->getManufacturer() ?: '');
                $cartItem->setSubcategory($this->getProductCategory($item->getProductId()));
                $cartItem->setProductCondition('NEW');
            } else {
                $cartItem->setCategory(strtoupper(StoreType::LABEL_OTHER));
            }

            $cartItem->setDescription($item->getName());
            $cartItem->setUnits(intval($item->getQtyOrdered()));
            $cartItem->setAmount($this->formatAmount($item->getPrice()));
            $cartItem->setCurrency(trim($order->getOrderCurrencyCode()));
            $cartItems[] = $this->convertDataToKeyValue($cartItem);
        }

        if ($shippingAmount > 0) {
            /** @var \Instant\Credit\Model\Api\Data\CartItemApi $cartItem */
            $cartItem = $this->cartItemFactory->create();
            $cartItem->setCategory(strtoupper(StoreType::LABEL_OTHER));
            $cartItem->setAmount($this->formatAmount($shippingAmount));
            $cartItem->setCurrency(trim($order->getOrderCurrencyCode()));
            $cartItem->setUnits(1);
            $cartItem->setDescription($order->getShippingDescription());

            $cartItems[] = $this->convertDataToKeyValue($cartItem);
        }

        return $cartItems;
    }

    /**
     * Update cart item to new version Cart to send in API
     * @param \Instant\Credit\Model\Api\Data\CartItemApi $cartItem
     * @return \Instant\Credit\Model\Api\Data\CartItemApi[]
     */
    private function convertDataToKeyValue($cartItem)
    {
        $arrData = $cartItem->getData();
        $arrResult = [];

        foreach ($arrData as $key => $value) {
            $keyFormatted = ucfirst(strtolower($key));
            /** @var \Instant\Credit\Model\Api\Data\CartItemApi $cartItem */
            $cartItem = $this->cartItemFactory->create();
            $cartItem->setKey($keyFormatted);
            $cartItem->setValue($value);

            $arrResult[] = $cartItem;
        }

        return $arrResult;
    }

    /**
     * @param $productId
     * @return string
     */
    protected function getProductCategory($productId)
    {
        /** @var \Magento\Catalog\Model\Product $product */
        $product = $this->productCollectionFactory->create()
            ->addFieldToFilter('entity_id', $productId)
            ->getFirstItem();

        $categoryStr = '';
        if ($product) {
            $productCategories = $product->getCategoryIds();
            if (count($productCategories) > 0) {
                /** @var \Magento\Catalog\Model\Category $category */
                $category = $this->categoryCollectionFactory->create()
                    ->addFieldToFilter('entity_id', $productCategories[0])
                    ->getFirstItem();

                if ($category) {
                    if ($category->getParentCategory()->getName()) {
                        $categoryStr = $category->getParentCategory()->getName();
                    } else {
                        $categoryStr = $category->getName() ?: '';
                    }
                }
            }
        } else {
            $this->logger->error('Can\'t find product with ID ' . $productId);
        }

        return $categoryStr;
    }

    /**
     * @param $customerId
     * @return \Instant\Credit\Model\Api\Data\HistoricOrderApi[]
     */
    protected function getHistoricOrders($customerId)
    {
        $historicOrderList = [];
        $orderCollection = $this->orderCollection->create()
            ->addFieldToFilter('status', ['nin' => ['canceled', 'pending', 'fraud']])
            ->addFieldToFilter('customer_id', $customerId)
            ->addFieldToFilter('created_at', ['gteq' => date('Y-m-d H:m:s', strtotime('-1 year'))]);

        if ($orderCollection->count()) {
            /** @var \Magento\Sales\Model\Order[] $orders */
            $orders = $orderCollection->getItems();

            foreach ($orders as $order) {
                /** @var \Instant\Credit\Model\Api\Data\HistoricOrderApi $historic */
                $historic = $this->historic->create();
                $historic->setStatus($this->getInstantCreditStatus($order->getStatus()));
                $historic->setDate($order->getCreatedAt());
                $historic->setAmount($this->formatAmount($order->getGrandTotal()));
                $historic->setCurrency($order->getOrderCurrencyCode());
                $historic->setCountry($order->getBillingAddress()->getCountryId());
                $historicOrderList[] = $historic;
            }

        }

        return $historicOrderList;
    }

    /**
     * Get province code from https://es.wikipedia.org/wiki/Anexo:Provincias_de_Espa%C3%B1a_por_c%C3%B3digo_postal
     * @param $province
     * @return string
     */
    protected function getProvinceCode($province)
    {
        $province = strtolower(trim($province));

        $provinces = [
            'a coruсa' => 15,
            'alava' => 1,
            'albacete' => 2,
            'alicante' => 3,
            'almeria' => 4,
            'asturias' => 33,
            'avila' => 5,
            'badajoz' => 6,
            'baleares' => 7,
            'barcelona' => 8,
            'burgos' => 9,
            'caceres' => 10,
            'cadiz' => 11,
            'cantabria' => 39,
            'castellon' => 12,
            'ceuta' => 51,
            'ciudad real' => 13,
            'cordoba' => 14,
            'cuenca' => 16,
            'girona' => 17,
            'granada' => 18,
            'guadalajara' => 19,
            'guipuzcoa' => 20,
            'huelva' => 21,
            'huesca' => 22,
            'jaen' => 23,
            'la rioja' => 26,
            'las palmas' => 35,
            'leon' => 24,
            'lleida' => 25,
            'lugo' => 27,
            'madrid' => 28,
            'malaga' => 29,
            'melilla' => 52,
            'murcia' => 30,
            'navarra' => 31,
            'ourense' => 32,
            'palencia' => 34,
            'pontevedra' => 36,
            'salamanca' => 37,
            'santa cruz de tenerife' => 38,
            'segovia' => 40,
            'sevilla' => 41,
            'soria' => 42,
            'tarragona' => 43,
            'teruel' => 44,
            'toledo' => 45,
            'valencia' => 46,
            'valladolid' => 47,
            'vizcaya' => 48,
            'zamora' => 49,
            'zaragoza' => 50,
        ];

        if (!isset($provinces[$province])) {
            return '';
        }

        return $provinces[$province];

    }


    /**
     * @param \Magento\Sales\Model\Order $order
     * @return \Instant\Credit\Model\Api\Data\EndUserApi
     */
    public function getEndUser(\Magento\Sales\Model\Order $order)
    {

        $billingAddress = $order->getBillingAddress();
        $shippingAddress = $order->getShippingAddress();

        // Get shipping if exists, if not, get billing address
        $principalAddress = ($shippingAddress) ?: $billingAddress;

        /** @var \Instant\Credit\Model\Api\Data\EndUserApi $endUser */
        $endUser = $this->endUser->create();

        $customer = null;
        $customerId = $order->getCustomerId();
        if ($customerId) {
            /** @var \Magento\Customer\Model\Customer $customer */
            $customer = $this->getCustomerById($customerId);
        }

        if($customer) {
            $endUser->setEndUserId($customer->getId());
            $endUser->setFirstName($customer->getFirstname());
            $endUser->setLastName($customer->getLastname() ?? '');
            $endUser->setPersonalId($customer->getTaxvat() ?? '');
            $endUser->setEmail($customer->getEmail());
            $endUser->setBirthDate($customer->getDob() ?? '');
            $endUser->setRegistrationDate($customer->getCreatedAt());
            $endUser->setHistoricOrders($this->getHistoricOrders($customer->getId())); // TODO: Get historic orders
        } else {
            $endUser->setEndUserId($order->getId());
            $endUser->setFirstName($order->getCustomerFirstname());
            $endUser->setLastName($order->getCustomerLastname() ?? '');
            $endUser->setPersonalId($order->getCustomerTaxvat() ?? '');
            $endUser->setEmail($order->getCustomerEmail());
            $endUser->setBirthDate($order->getCustomerDob() ?? '');
            $endUser->setRegistrationDate(date('Y-m-d H:i:s'));
        }

        $endUser->setMobile($principalAddress->getTelephone());
        $endUser->setAlternativePhoneNumber('');
        $endUser->setIp($this->remoteAddress->getRemoteAddress());

        $endUser->setAddress($this->getAddress($principalAddress));

        return $endUser;
    }

    /**
     * @param \Magento\Sales\Api\Data\OrderAddressInterface $addressOrder
     * @return \Instant\Credit\Model\Api\Data\AddressApi
     */
    protected function getAddress(\Magento\Sales\Api\Data\OrderAddressInterface $addressOrder)
    {
        /** @var \Instant\Credit\Model\Api\Data\AddressApi $icAddress */
        $icAddress = $this->address->create()
            ->setStreet($addressOrder->getStreet()[0])
            ->setPostalCode($addressOrder->getPostCode())
            ->setProvince($this->getProvinceCode($addressOrder->getRegion()))
            ->setCity($addressOrder->getCity())
            ->setCountry($addressOrder->getCountryId());

        return $icAddress;
    }

    /**
     * @param $path
     * @return \Magento\Framework\Controller\ResultInterface
     */
    private function redirect($path)
    {
        $redirect = $this->resultFactory->create(\Magento\Framework\Controller\ResultFactory::TYPE_REDIRECT);
        $redirect->setUrl($path);
        return $redirect;
    }

    /**
     * @param $path
     * @return mixed
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    private function getUrl($path)
    {
        return $this->storeManager->getStore()->getUrl($path);
    }


    /**
     * @param $data
     * @return string
     */
    public function getSignature($data)
    {
        $secretKey = $this->scopeConfig->getValue('payment/instantcredit/secret_key');
        $data = $this->serializer->serialize($data);
        return hash('sha256', $secretKey . $data);
    }

    /**
     * Format amount to send to API
     * @param $amount
     * @return string
     */
    private function formatAmount($amount)
    {
        return number_format($amount, 2, '.', '');
    }
}
