<?php

namespace Instant\Credit\Controller\Success;

use Magento\Framework\App\ActionInterface;
use Magento\Framework\App\ResponseInterface;

/**
 * Class Index
 * @package Instant\Credit\Controller\Success
 * @author Orenyi
 */
class Index implements ActionInterface
{

    /**
     * @var \Magento\Framework\App\RequestInterface
     */
    private $request;
    /**
     * @var \Magento\Checkout\Model\Session
     */
    private $checkoutSession;
    /**
     * @var \Magento\Framework\View\Result\PageFactory
     */
    private $pageFactory;
    /**
     * @var \Magento\Framework\Controller\ResultFactory
     */
    private $resultFactory;
    /**
     * @var \Magento\Sales\Model\OrderRepository
     */
    private $orderRepository;
    /**
     * @var \Magento\Framework\Message\ManagerInterface
     */
    private $messageManager;

    /**
     * Success constructor.
     * @param \Magento\Framework\App\RequestInterface $request
     * @param \Magento\Checkout\Model\Session $checkoutSession
     * @param \Magento\Framework\View\Result\PageFactory $pageFactory
     * @param \Magento\Framework\Controller\ResultFactory $resultFactory
     * @param \Magento\Sales\Model\OrderRepository $orderRepository
     * @param \Magento\Framework\Message\ManagerInterface $messageManager
     */
    public function __construct(
        \Magento\Framework\App\RequestInterface $request,
        \Magento\Checkout\Model\Session $checkoutSession,
        \Magento\Framework\View\Result\PageFactory $pageFactory,
        \Magento\Framework\Controller\ResultFactory $resultFactory,
        \Magento\Sales\Model\OrderRepository $orderRepository,
        \Magento\Framework\Message\ManagerInterface $messageManager
    ) {

        $this->request = $request;
        $this->checkoutSession = $checkoutSession;
        $this->pageFactory = $pageFactory;
        $this->resultFactory = $resultFactory;
        $this->orderRepository = $orderRepository;
        $this->messageManager = $messageManager;
    }

    /**
     * Execute action based on request and return result
     *
     * @throws \Magento\Framework\Exception\AlreadyExistsException
     * @throws \Magento\Framework\Exception\InputException
     * @throws \Magento\Framework\Exception\NoSuchEntityException
     */
    public function execute()
    {
        $order = $this->checkoutSession->getLastRealOrder();
        $redirect = $this->resultFactory->create(\Magento\Framework\Controller\ResultFactory::TYPE_REDIRECT);

        if ($order->getData()) {
            $order->addCommentToStatusHistory(__('Instant Credit Redirection: Success page'));
            $this->orderRepository->save($order);
        }
        $this->messageManager->addSuccessMessage(__('Your order has been processed and confirmed successfully.'));

        /** @var \Magento\Framework\Controller\ResultFactory $redirect */
        $redirect->setUrl('/checkout/onepage/success');
        return $redirect;
    }
}