<?php

namespace Instant\Credit\Helper;

use Magento\Framework\App\Helper\AbstractHelper;

/**
 * Class IcHelper
 * @package Instant\Credit\Helper
 * @author Orenyi
 */
class IcHelper extends AbstractHelper
{
    /**
     * Get if payment method is enabled from Instant Credit config
     * @return mixed
     */
    public function getIsEnabled()
    {
        return $this->scopeConfig->getValue('payment/instantcredit/active', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
    }

    /**
     * Get Client ID value from Instant Credit config
     * @return mixed
     */
    public function getClientId()
    {
        return $this->scopeConfig->getValue('payment/instantcredit/client_id', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
    }

    /**
     * Get Secret Key value from Instant Credit config
     * @return mixed
     */
    public function getSecretKey()
    {
        return $this->scopeConfig->getValue('payment/instantcredit/secret_key', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
    }

    /**
     * Get Username value from Instant Credit config
     * @return mixed
     */
    public function getUsername()
    {
        return $this->scopeConfig->getValue('payment/instantcredit/username', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
    }

    /**
     * Get Password value from Instant Credit config
     * @return mixed
     */
    public function getPassword()
    {
        return $this->scopeConfig->getValue('payment/instantcredit/password', \Magento\Store\Model\ScopeInterface::SCOPE_STORE);
    }

   /**
     * Get upper limit to show payment method from Instant Credit config
     * @return float
     */
    public function getUpperLimit()
    {
        return floatval($this->scopeConfig->getValue('payment/instantcredit/ic_prices/upper_limit', \Magento\Store\Model\ScopeInterface::SCOPE_STORE));
    }

    /**
     * Get lower limit to show payment method from Instant Credit config
     * @return float
     */
    public function getLowerLimit()
    {
        return floatval($this->scopeConfig->getValue('payment/instantcredit/ic_prices/lower_limit', \Magento\Store\Model\ScopeInterface::SCOPE_STORE));
    }

    /**
     * Check if price passed by parameter is between limits
     * @param $price
     * @return bool
     */
    public function isBetweenLimits($price)
    {
        $price = floatval($price);
        $upper = $this->getUpperLimit();
        $lower = $this->getLowerLimit();

        if ($upper === 0 && $lower === 0) {
            return true;
        }

        return ($price >= $lower && $price <= $upper);
    }
}