<?php

namespace Instant\Credit\Model\Api\Data;

use Instant\Credit\Api\Data\TransactionsInterface;
use Magento\Framework\DataObject\IdentityInterface;
use Magento\Framework\Model\AbstractExtensibleModel;
use \Instant\Credit\Model\ResourceModel\Transactions as ResourceTransactions;

/**
 * Class Transactions
 * @package Instant\Credit\Model\Api\Data
 * @author Orenyi
 */
class Transactions extends AbstractExtensibleModel implements TransactionsInterface, IdentityInterface
{

    const CACHE_TAG = 'instantcredit_transactions';

    /**
     * @var string
     */
    protected $_cacheTag = 'instantcredit_transactions';

    /**
     * @var string
     */
    protected $_eventPrefix = 'instantcredit_transactions';

    /**
     * @inheritdoc
     */
    public function _construct()
    {
        $this->_init(ResourceTransactions::class);
    }

    /**
     * @return mixed
     */
    public function getTransactionId()
    {
        return $this->_getData(self::ID);
    }

    /**
     * @return mixed
     */
    public function getOrderId()
    {
        return $this->_getData(self::ORDER_ID);
    }

    /**
     * @return mixed
     */
    public function getCustom()
    {
        return $this->_getData(self::CUSTOM);
    }

    /**
     * @return mixed
     */
    public function getResult()
    {
        return $this->_getData(self::RESULT);
    }

    /**
     * @return mixed
     */
    public function getResultType()
    {
        return $this->_getData(self::RESULT_TYPE);
    }

    /**
     * @return mixed
     */
    public function getCountry()
    {
        return $this->_getData(self::COUNTRY);
    }

    /**
     * @return mixed
     */
    public function getAmount()
    {
        return $this->_getData(self::AMOUNT);
    }

    /**
     * @return mixed
     */
    public function getCurrency()
    {
        return $this->_getData(self::CURRENCY);
    }

    /**
     * @return mixed
     */
    public function getSignature()
    {
        return $this->_getData(self::SIGNATURE);
    }

    /**
     * @return mixed
     */
    public function getConfirmed()
    {
        return $this->_getData(self::CONFIRMED);
    }

    /**
     * @return mixed
     */
    public function getDescription()
    {
        return $this->_getData(self::DESCRIPTION);
    }

    /**
     * @return mixed
     */
    public function getCreatedAt()
    {
        return $this->_getData(self::CREATED_AT);
    }

    /**
     * @return mixed
     */
    public function getUpdatedAt()
    {
        return $this->_getData(self::UPDATED_AT);
    }

    /**
     * @param int $transactionId
     * @return mixed
     */
    public function setTransactionId(int $transactionId)
    {
        return $this->setData(self::ID, $transactionId);
    }

    /**
     * @param int $orderId
     * @return mixed
     */
    public function setOrderId(string $orderId)
    {
        return $this->setData(self::ORDER_ID, $orderId);
    }

    /**
     * @param string $custom
     * @return mixed
     */
    public function setCustom(string $custom)
    {
        return $this->setData(self::CUSTOM, $custom);
    }

    /**
     * @param string $result
     * @return mixed
     */
    public function setResult(string $result)
    {
        return $this->setData(self::RESULT, $result);
    }

    /**
     * @param string $resultType
     * @return mixed
     */
    public function setResultType(string $resultType)
    {
        return $this->setData(self::RESULT_TYPE, $resultType);
    }

    /**
     * @param string $country
     * @return mixed
     */
    public function setCountry(string $country)
    {
        return $this->setData(self::COUNTRY, $country);
    }

    /**
     * @param float $amount
     * @return mixed
     */
    public function setAmount(float $amount)
    {
        return $this->setData(self::AMOUNT, $amount);
    }

    /**
     * @param string $currency
     * @return mixed
     */
    public function setCurrency(string $currency)
    {
        return $this->setData(self::CURRENCY, $currency);
    }

    /**
     * @param string $signature
     * @return mixed
     */
    public function setSignature(string $signature)
    {
        return $this->setData(self::SIGNATURE, $signature);
    }

    /**
     * @param bool $confirmed
     * @return mixed
     */
    public function setConfirmed(bool $confirmed)
    {
        return $this->setData(self::CONFIRMED, $confirmed);
    }

    /**
     * @param string $description
     * @return mixed
     */
    public function setDescription(string $description)
    {
        return $this->setData(self::DESCRIPTION, $description);
    }

    /**
     * Return unique ID(s) for each object in system
     *
     * @return string[]
     */
    public function getIdentities()
    {
        return [self::CACHE_TAG . '_' . $this->getId()];
    }

}