<?php

namespace Instant\Credit\Model\Config\Backend;

use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Validator\Url as UrlValidator;
use Magento\Framework\App\ObjectManager;

/**
 * Class Limit
 * @package Instant\Credit\Model\Config\Backend
 * @author Orenyi
 */
class Limit extends \Magento\Framework\App\Config\Value
{
    /**
     * @var \Magento\Framework\App\Config\ScopeConfigInterface
     */
    private $config;


    /**
     * @param \Magento\Framework\Model\Context $context
     * @param \Magento\Framework\Registry $registry
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $config
     * @param \Magento\Framework\App\Cache\TypeListInterface $cacheTypeList
     * @param \Magento\Framework\Model\ResourceModel\AbstractResource $resource
     * @param \Magento\Framework\Data\Collection\AbstractDb $resourceCollection
     * @param array $data
     */
    public function __construct(
        \Magento\Framework\Model\Context $context,
        \Magento\Framework\Registry $registry,
        \Magento\Framework\App\Config\ScopeConfigInterface $config,
        \Magento\Framework\App\Cache\TypeListInterface $cacheTypeList,
        \Magento\Framework\Model\ResourceModel\AbstractResource $resource = null,
        \Magento\Framework\Data\Collection\AbstractDb $resourceCollection = null,
        array $data = []
    ) {
        $this->config = $config;
        parent::__construct($context, $registry, $config, $cacheTypeList, $resource, $resourceCollection, $data);
    }

    /**
     * @return void
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function beforeSave()
    {
        $lowerLimit = floatval($this->getFieldsetDataValue('lower_limit'));
        $upperLimit = floatval($this->getFieldsetDataValue('upper_limit'));

        $limit = floatval($this->getValue());

        if ($this->getField() === 'upper_limit' || $this->getField() === 'lower_limit') {
            if (!is_numeric($this->getValue())) {
                throw new LocalizedException(__('Upper limit must be a numeric value'));
            }
        }

        if ($this->getField() === 'upper_limit') {
            if ($limit < $lowerLimit) {
                throw new LocalizedException(__('Upper limit must be higher than lower limit'));
            }
        } else if ($this->getField() === 'lower_limit') {
            if ($limit > $upperLimit) {
                throw new LocalizedException(__('Lower limit must be lower than upper limit'));
            }
        }
    }
}
