<?php

namespace Instant\Credit\Observer;

use Instant\Credit\Model\InstantCreditMethod;
use Magento\Framework\Event\ObserverInterface;

/**
 * Class PaymentMethodAvailable
 * @package Instant\Credit\Observer
 * @author Orenyi
 */
class PaymentMethodAvailable implements ObserverInterface
{

    /**
     * @var \Instant\Credit\Helper\IcHelper
     */
    private $icHelper;

    /**
     * PaymentMethodAvailable constructor.
     * @param \Instant\Credit\Helper\IcHelper $icHelper
     */
    public function __construct(
        \Instant\Credit\Helper\IcHelper $icHelper
    ) {
        $this->icHelper = $icHelper;
    }

    /**
     * @param \Magento\Framework\Event\Observer $observer
     */
    public function execute(\Magento\Framework\Event\Observer $observer)
    {
        if($observer->getEvent()->getMethodInstance()->getCode()== InstantCreditMethod::INSTANTCREDIT_METHOD){
            $grandTotal = $observer->getQuote()->getGrandTotal();

            if (!$this->icHelper->getIsEnabled() || !$this->filledConfigValues() || !$this->icHelper->isBetweenLimits($grandTotal)) {
                $checkResult = $observer->getEvent()->getResult();
                $checkResult->setData('is_available', false); // Disable payment method at checkout page
            }
        }
    }

    /**
     * @param $value
     * @return bool
     */
    protected function hasValue($value)
    {
        return $value !== null && $value !== '';
    }

    /**
     * @return bool
     */
    protected function filledConfigValues()
    {
        return $this->hasValue($this->icHelper->getSecretKey()) &&
            $this->hasValue($this->icHelper->getClientId()) &&
            $this->hasValue($this->icHelper->getUsername()) &&
            $this->hasValue($this->icHelper->getPassword());
    }
}