<?php

namespace Instant\Credit\Setup;

use Instant\Credit\Api\Data\TransactionsInterface;

/**
 * Class InstallSchema
 * @package Instant\Credit\Setup
 * @author Orenyi
 */
class InstallSchema implements \Magento\Framework\Setup\InstallSchemaInterface
{
    /**
     * @param \Magento\Framework\Setup\SchemaSetupInterface $setup
     * @param \Magento\Framework\Setup\ModuleContextInterface $context
     * @throws \Zend_Db_Exception
     */
    public function install(
        \Magento\Framework\Setup\SchemaSetupInterface $setup,
        \Magento\Framework\Setup\ModuleContextInterface $context
    ) {
        $this->createTransactionsTable($setup);
    }

    /**
     * @param \Magento\Framework\Setup\SchemaSetupInterface $setup
     * @throws \Zend_Db_Exception
     */
    private function createTransactionsTable(\Magento\Framework\Setup\SchemaSetupInterface $setup)
    {
        $installer = $setup;

        $installer->startSetup();
        if (!$installer->tableExists(TransactionsInterface::TABLE)) {
            $table = $installer->getConnection()->newTable(
                $installer->getTable(TransactionsInterface::TABLE)
            )
                ->addColumn(
                    TransactionsInterface::ENTITY_ID,
                    \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                    null,
                    [
                        'identity' => true,
                        'nullable' => false,
                        'primary'  => true,
                        'unsigned' => true,
                    ],
                    'Entity ID'
                )
                ->addColumn(
                    TransactionsInterface::ID,
                    \Magento\Framework\DB\Ddl\Table::TYPE_INTEGER,
                    null,
                    [
                        'nullable' => false,
                        'unique' => true
                    ],
                    'Transaction ID'
                )
                ->addColumn(
                    TransactionsInterface::ORDER_ID,
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    20,
                    ['nullable => false'],
                    'Order ID'
                )
                ->addColumn(
                    TransactionsInterface::CUSTOM,
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    255,
                    [],
                    'Custom content'
                )
                ->addColumn(
                    TransactionsInterface::RESULT,
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    10,
                    ['nullable => false'],
                    'Result Transaction'
                )
                ->addColumn(
                    TransactionsInterface::RESULT_TYPE,
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    50,
                    ['nullable => false'],
                    'Result Type Transaction'
                )
                ->addColumn(
                    TransactionsInterface::COUNTRY,
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    10,
                    ['nullable => false'],
                    'Country'
                )
                ->addColumn(
                    TransactionsInterface::AMOUNT,
                    \Magento\Framework\DB\Ddl\Table::TYPE_DECIMAL,
                    '12,4',
                    ['nullable => false'],
                    'Amount Transaction'
                )
                ->addColumn(
                    'currency',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    10,
                    ['nullable => false'],
                    'Currency'
                )
                ->addColumn(
                    TransactionsInterface::SIGNATURE,
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    200,
                    ['nullable => false'],
                    'Signature Transaction'
                )
                ->addColumn(
                    TransactionsInterface::CONFIRMED,
                    \Magento\Framework\DB\Ddl\Table::TYPE_BOOLEAN,
                    1,
                    [
                        'nullable => false',
                        'default' => 0
                    ],
                    'Is transaction confirmed?'
                )
                ->addColumn(
                    'description',
                    \Magento\Framework\DB\Ddl\Table::TYPE_TEXT,
                    200,
                    ['nullable => false'],
                    'Result description'
                )
                ->addColumn(
                    TransactionsInterface::CREATED_AT,
                    \Magento\Framework\DB\Ddl\Table::TYPE_TIMESTAMP,
                    null,
                    ['nullable' => false, 'default' => \Magento\Framework\DB\Ddl\Table::TIMESTAMP_INIT],
                    'Created At'
                )->addColumn(
                    TransactionsInterface::UPDATED_AT,
                    \Magento\Framework\DB\Ddl\Table::TYPE_TIMESTAMP,
                    null,
                    ['nullable' => false, 'default' => \Magento\Framework\DB\Ddl\Table::TIMESTAMP_INIT_UPDATE],
                    'Updated At')
                ->setComment('Post Table');


            $installer->getConnection()->createTable($table);
        }
        $installer->endSetup();
    }
}