<?php

namespace Instant\Credit\ViewModel;

use Instant\Credit\Api\Data\TransactionsInterface;
use Instant\Credit\Model\InstantCreditMethod;

/**
 * Class OrderAdminhtmlViewModel
 * @package Instant\Credit\ViewModel
 * @author Orenyi
 */
class OrderAdminhtmlViewModel implements \Magento\Framework\View\Element\Block\ArgumentInterface
{
    /**
     * @var \Magento\Framework\Registry
     */
    private $registry;
    /**
     * @var \Instant\Credit\Model\ResourceModel\Transactions\CollectionFactory
     */
    private $transactionCollection;

    /**
     * OrderAdminhtmlViewModel constructor.
     * @param \Magento\Framework\Registry $registry
     * @param \Instant\Credit\Model\ResourceModel\Transactions\CollectionFactory $transactionCollection
     */
    public function __construct(
        \Magento\Framework\Registry $registry,
        \Instant\Credit\Model\ResourceModel\Transactions\CollectionFactory $transactionCollection
    ) {
        $this->registry = $registry;
        $this->transactionCollection = $transactionCollection;
    }

    /**
     * Get current order
     * @return \Magento\Sales\Model\Order
     */
    public function getOrder()
    {
        return $this->registry->registry('current_order');
    }

    /**
     * Return if payment method used is Instant Credit
     * @return bool
     */
    public function isInstantCredit()
    {
        /** @var \Magento\Sales\Model\Order $order */
        $order = $this->getOrder();

        if ($order) {
            return $order->getPayment()->getMethod() === InstantCreditMethod::INSTANTCREDIT_METHOD;
        }

        return false;
    }

    /**
     * Return transaction of current product
     * @return \Instant\Credit\Model\Api\Data\Transactions
     */
    public function getTransaction()
    {
        $order = $this->getOrder();
        if ($order) {
            $orderId = $order->getIncrementId();
            /** @var \Instant\Credit\Model\ResourceModel\Transactions\Collection $transactionCol */
            $transactionCol = $this->transactionCollection->create();
            $transactionCol->addFieldToFilter(TransactionsInterface::ORDER_ID, $orderId);

            if ($transactionCol->count() > 0) {
                /** @var \Instant\Credit\Model\Api\Data\Transactions $item */
                $item = $transactionCol->getFirstItem();
                return $item;
            }
        }

        return null;
    }
}